@@ -8,6 +8,8 @@ import phylotreelib as pt
 
 import numpy as np
 import pandas as pd
+
+# Python note: I could make these imports conditional (placing them in MBASR section)
 import rpy2
 import rpy2.robjects as robjects
 from rpy2.robjects.packages import importr
@@ -23,114 +25,73 @@ class AncError(Exception):
 ###################################################################################################
 ###################################################################################################
 
-class Anc_recon():
-    """Objects of this class contain a tree and a corresponding set of sequences.
+class AncRecon:
+    """Created from one ancseq object, and one anctrait object.
+    Contains info about tree, and also about sequences and traits (ancestral and reconstructed)
 
-    Some sequences correspond to leaves on the tree. These are observed sequences.
-    Other sequences correspond to internal nodes on the tree. These are ancestral reconstructions.
+    Some seqs/trait correspond to leaves on the tree. These are observed seqs/trait.
+    Other seqs/trait correspond to internal nodes. These are ancestral reconstructions.
 
-    There is also information about credibility of sequence states: 100% for leaves, possibly
-    less for internal nodes. If there is no information about credibility, then these are
-    set to 100%.
+    There may also be information about credibility of reconstructed seqs/traits
+    (100% for leaves, possibly less for internal nodes)
 
-    Objects may also contain information about other, non-sequence, traits. These also have
-    associated credibilities.
-
-    There are several constructors corresponding to different sources of information.
-    There are also methods for adding extra information to already constructed objects.
+    Object can output information about nodes, branches, changes, etc.
     """
 
-    def __init__(self):
-        self.tree = None
-        self.alignment = None
-        self.seqprob = None
-        self.trait = {}
-        self.traitprob = {}
-        self.sortedintnodes = None
-        self.sortednodes = None
+    def __init__(self, ancseq, anctrait):
+        self.tree = ancseq.tree
+        self.alignment = ancseq.alignment
+        if hasattr(ancseq, "seqprob"):
+            self.seqprob = ancseq.seqprob
+        else:
+            self.seqprob = None # Python note: should I set to zeroes?
+        self.sortedintnodes = sorted(list(self.tree.intnodes))
+        self.sortednodes = sorted(list(self.tree.leaves))
+        self.sortednodes.extend(self.sortedintnodes)
+
+        trait_tree = anctrait.tree
+        orig_traitdict = anctrait.traitdict
+        orig_traitprob = anctrait.traitprob    # Python note: Always there?
+
+        self.traitdict, self.traitprob = self._match_traitid_seqid(self.tree, trait_tree,
+                                                            orig_traitdict, orig_traitprob)
+
+    ###########################################################################################
+
+    def _match_traitid_seqid(self, seqtree, trait_tree, traitdict, traitprob):
+        trateid2seqid,unmatch1,unmatch2 = trait_tree.match_nodes(seqtree)
+        if (unmatch1 != None) or (unmatch2 != None):
+            raise AncError("Seq-tree and trait-tree are rooted differently. Cannot merge info")
+        new_traitdict = {}
+        new_traitprob = {}
+        for trateid,seqid in trateid2seqid.items():
+            new_traitdict[seqid] = traitdict[trateid]
+            new_traitprob[seqid] = traitprob[trateid]
+        return new_traitdict, new_traitprob
 
     ###############################################################################################
 
-    @classmethod
-    def from_baseml_mbasr(cls, baseml_rstfile,
-                              mbasr_treefile, mbasr_intnodefile, mbasr_leafstatefile,
-                              state0="state_0", state1="state_1"):
-        """Combine information from BASEML rst file, and MBASR ancestral state reconstruction
-
-        The following attributes are added:
-            tree
-            alignment of sequences, including ancestral reconstructions. Seqname = str(nodeid)
-            probabilities for seq residues (dict of nodeid:numpy array)
-            trait state (dict of nodeid:state)
-            trait state probabillity (dict of nodeid:state probability)
-
-        The nodeids used by MBASR are changed to match those used by BASEML
-        """
-
-        obj = cls()
-        obj.__init__()
-        ba_file = _Baseml_rstfile(baseml_rstfile)
-        obj.tree = ba_file.get_tree()
-        obj.alignment, obj.seqprob = ba_file.get_alignment()
-
-        mb_file = _MBASR_file(mbasr_treefile, mbasr_intnodefile, mbasr_leafstatefile, state0, state1)
-        mb_tree = mb_file.get_tree()
-        mb_trait, mb_probs = mb_file.get_trait_dict()
-        obj.sortedintnodes = sorted(list(obj.tree.intnodes))
-        obj.sortednodes = sorted(list(obj.tree.leaves))
-        obj.sortednodes.extend(obj.sortedintnodes)
-
-        mbnode2banode, unmatch_baseml, unmatch_mbasr = mb_tree.match_nodes(obj.tree)
-        if (unmatch_baseml != None) or (unmatch_mbasr != None):
-            raise AncError("BASEML and MBASR trees are rooted differently. Cannot merge info")
-        for mbnode,banode in mbnode2banode.items():
-            obj.trait[banode] = mb_trait[mbnode]
-            obj.traitprob[banode] = mb_probs[mbnode]
-        return obj
-
-    ###############################################################################################
-
-    @classmethod
-    def from_timetree(cls, ancseqfile, seqtreefile, statetreefile, stateprobfile):
-        """Parse information from timetree ancestral reconstruction of sequences and state
-
-        The following attributes are added:
-            tree
-            alignment of sequences, including ancestral reconstructions. Seqname = str(nodeid)
-            trait state (dict of nodeid:state)
-
-        While keeping track of the internal node IDs used in the two trees (probably identical...)
-        """
-
-        obj = cls()
-        obj.__init__()
-        tt = _TimeTreeResults(ancseqfile, seqtreefile, statetreefile, stateprobfile)
-        obj.tree = tt.seqtree
-        obj.alignment = tt.alignment
-        obj.trait = tt.trait
-        obj.traitprob = tt.traitprob
-        obj.sortedintnodes = sorted(list(obj.tree.intnodes))
-        obj.sortednodes = sorted(list(obj.tree.leaves))
-        obj.sortednodes.extend(obj.sortedintnodes)
-        return obj
-
-    ###############################################################################################
-
-    def nodeinfo(self, outfilename, varseq=False, poslist=None, zeroindex=True):
+    def nodeinfo(self, outfilename, varseq=False, poslist=None, zeroindex=True, probmin=None,
+                 translate=False):
         """Writes results to 'outfilename':
         Output is one line per node in tree - with following informations:
-            nodeid  traitstate  seqstates
+            nodeid  traitstate  traitprob seqstates
         option varseq=True outputs only variable sites from sequences.
+        option probmin (if not None): Only print info for nodes with traitprob > probmin
         Can also explicitly provide poslist of sites to be printed. Indexing in poslist
         can start at 0 (zeroindex=True) or 1 (zeroindex=False)"""
 
         with open(outfilename, "w") as outfile:
-            outfile.write("# {}\t{}\t{}\n".format("nodeid", "trait", "seq"))
+            outfile.write("# {}\t{}\t{}\t{}\n".format("nodeid", "trait", "traitprob", "seq"))
             pos = None
             if varseq and poslist:
                 raise AncError("Specify either varseq or poslist option - not both")
+            if translate:
+                alignment = self.alignment.translate()
+            else:
+                alignment = self.alignment
             if varseq:
-                pos = self.alignment.varcols()
+                pos = alignment.varcols()
             elif poslist:
                 pos = poslist
 
@@ -138,35 +99,45 @@ class Anc_recon():
                 pos = [p - 1 for p in pos]
 
             for nodeid in self.sortednodes:
-                seqname = str(nodeid)
-                trait = self.trait[nodeid]
-                if pos:
-                    seq = self.alignment.getseq(seqname).subseqpos(pos).seq
-                else:
-                    seq = self.alignment.getseq(seqname).seq
-                outfile.write("{}\t{}\t{}\n".format(nodeid, trait, seq))
+                if (not probmin) or (self.traitprob[nodeid] > probmin):
+                    seqname = str(nodeid)
+                    trait = self.traitdict[nodeid]
+                    traitprob = self.traitprob[nodeid]
+                    if pos:
+                        seq = alignment.getseq(seqname).subseqpos(pos).seq
+                    else:
+                        seq = alignment.getseq(seqname).seq
+                    outfile.write(f"{nodeid}\t{trait}\t{traitprob:.3f}\t{seq}\n")
 
     ###############################################################################################
 
     def branchinfo(self, outfilename, varseq=False, poslist=None, zeroindex=True,
-                    printif_traitdiff=False, printif_seqdiff=False):
+                    printif_traitdiff=False, printif_seqdiff=False, probmin=None,
+                    translate=False):
         """Writes results to 'outfilename':
-                    Outputs one line per branch with following informations:
-            nodeid_from   nodeid_to   trait_from   trait_to   seq_from   seq_to
+        Outputs one line per branch with following informations:
+          nodeid_from  nodeid_to  trait_from  trait_to  traitprob_from  traitprob_to seq_from  seq_to
         Option varseq=True (default) output only variable sites from sequences.
         Option poslist: specify sites to be printed.
         Option zeroindex=True: Start indexing of poslist at 0 (otherwise start at 1)
         Option printif_traitdiff=True: only print branches where traits differ
-        Option printif_seqdiff=True: only print branches where selected residues have changed"""
+        Option printif_seqdiff=True: only print branches where selected residues have changed
+        option probmin (if not None): Only print info where both nodes have traitprob > probmin
+        """
 
         with open(outfilename, "w") as outfile:
-            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\n".format(
-                   "node_from", "node_to", "trait_from", "trait_to", "seq_from", "seq_to"))
+            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
+                   "node_from", "node_to", "trait_from", "trait_to",
+                   "traitprob_from", "traitprob_to", "seq_from", "seq_to"))
             pos = None
             if varseq and poslist:
                 raise AncError("Specify either varseq or poslist option - not both")
+            if translate:
+                alignment = self.alignment.translate()
+            else:
+                alignment = self.alignment
             if varseq:
-                pos = self.alignment.varcols()
+                pos = alignment.varcols()
             elif poslist:
                 pos = poslist
 
@@ -175,71 +146,81 @@ class Anc_recon():
 
             for nodefrom in self.sortedintnodes:
                 for nodeto in self.tree.children(nodefrom):
-                    seqnamefrom = str(nodefrom)
-                    seqnameto = str(nodeto)
-                    traitfrom = self.trait[nodefrom]
-                    traitto = self.trait[nodeto]
-                    if pos:
-                        seqfrom = self.alignment.getseq(seqnamefrom).subseqpos(pos).seq
-                        seqto = self.alignment.getseq(seqnameto).subseqpos(pos).seq
-                    else:
-                        seqfrom = self.alignment.getseq(seqnamefrom).seq
-                        seqto = self.alignment.getseq(seqnameto).seq
+                    if (not probmin) or (
+                            (self.traitprob[nodefrom] > probmin) and
+                            (self.traitprob[nodeto] > probmin)):
+                        seqnamefrom = str(nodefrom)
+                        seqnameto = str(nodeto)
+                        traitfrom = self.traitdict[nodefrom]
+                        traitto = self.traitdict[nodeto]
+                        traitprobfrom = self.traitprob[nodefrom]
+                        traitprobto = self.traitprob[nodeto]
+                        if pos:
+                            seqfrom = alignment.getseq(seqnamefrom).subseqpos(pos).seq
+                            seqto = alignment.getseq(seqnameto).subseqpos(pos).seq
+                        else:
+                            seqfrom = alignment.getseq(seqnamefrom).seq
+                            seqto = alignment.getseq(seqnameto).seq
 
-                    # Could test for traitdiff before setting seq, but neater code this way...
-                    printbranch = True
-                    if printif_traitdiff and (traitfrom==traitto):
-                        printbranch = False
-                    if printif_seqdiff and (seqfrom==seqto):
-                        printbranch = False
-                    if printbranch:
-                        outfile.write("{}\t{}\t{}\t{}\t{}\t{}\n".format(
-                               nodefrom, nodeto, traitfrom, traitto, seqfrom, seqto))
+                        # Could test for traitdiff before setting seq, but neater code this way...
+                        printbranch = True
+                        if printif_traitdiff and (traitfrom==traitto):
+                            printbranch = False
+                        if printif_seqdiff and (seqfrom==seqto):
+                            printbranch = False
+                        if printbranch:
+                            outfile.write("{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\n".format(
+                                   nodefrom, nodeto, traitfrom, traitto,
+                                   traitprobfrom, traitprobto, seqfrom, seqto))
 
     ###############################################################################################
 
-    def branchdiff(self, outfilename, zeroindex=True, printif_traitdiff=False):
+    def branchdiff(self, outfilename, zeroindex=True, printif_traitdiff=False, probmin=None,
+                   translate=False):
         """Writes results to 'outfilename':
         Output is one line of output for each sequence change, on all branches of the tree:
-            node_from   node_to   site  trait_from   trait_to   residue_from   residue_to
+            node_from, node_to, site, trait_from, trait_to,
+                                      traitprob_from, traitprob_to, residue_from, residue_to
         Here 'site' is the index of the sequence residue.
         Option zeroindex=False causes numbering to start at 1 (otherwise at 0)
         Option printif_traitdiff=True: only print branches where traits differ
+        Option probmin (if not None): Only print info where both nodes have traitprob > probmin
         """
 
         with open(outfilename, "w") as outfile:
-            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
-               "node_from", "node_to", "seqpos", "trait_from", "trait_to", "residue_from", "residue_to"))
+            outfile.write("# {}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
+               "node_from", "node_to", "seqpos", "trait_from", "trait_to",
+               "traitprob_from", "traitprob_to", "residue_from", "residue_to"))
+            if translate:
+                alignment = self.alignment.translate()
+            else:
+                alignment = self.alignment
             for nodefrom in self.sortedintnodes:
                 for nodeto in self.tree.children(nodefrom):
-                    seqnamefrom = str(nodefrom)
-                    seqnameto = str(nodeto)
-                    traitfrom = self.trait[nodefrom]
-                    traitto = self.trait[nodeto]
-                    seqfrom = self.alignment.getseq(seqnamefrom)
-                    seqto = self.alignment.getseq(seqnameto)
-                    difflist = seqfrom.seqdiff(seqto, zeroindex)
+                    if (not probmin) or (
+                            (self.traitprob[nodefrom] > probmin) and
+                            (self.traitprob[nodeto] > probmin)):
+                        seqnamefrom = str(nodefrom)
+                        seqnameto = str(nodeto)
+                        traitfrom = self.traitdict[nodefrom]
+                        traitto = self.traitdict[nodeto]
+                        traitprobfrom = self.traitprob[nodefrom]
+                        traitprobto = self.traitprob[nodeto]
+                        seqfrom = alignment.getseq(seqnamefrom)
+                        seqto = alignment.getseq(seqnameto)
+                        difflist = seqfrom.seqdiff(seqto, zeroindex)
 
-                    # Could test for traitdiff before setting seq, but neater code this way...
-                    if (not printif_traitdiff) or (traitfrom!=traitto):
-                        for site,resfrom,resto in difflist:
-                            outfile.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(
-                               nodefrom, nodeto, site, traitfrom, traitto, resfrom, resto))
-
-    ###############################################################################################
-
-    def varpos(self, zeroindex=True):
-        """Returns list of variable (unconserved) sites in alignment of sequences"""
-
-        varpos = self.alignment.varcols()
-        if not zeroindex:
-            varpos = [pos+1 for pos in varpos]
-        return varpos
+                        # Could test for traitdiff before setting seq, but neater code this way...
+                        if (not printif_traitdiff) or (traitfrom!=traitto):
+                            for site,resfrom,resto in difflist:
+                                outfile.write("{}\t{}\t{}\t{}\t{}\t{:.3f}\t{:.3f}\t{}\t{}\n".format(
+                                   nodefrom, nodeto, site, traitfrom, traitto,
+                                   traitprobfrom, traitprobto, resfrom, resto))
 
 ###################################################################################################
 ###################################################################################################
 
-class _Baseml_rstfile():
+class BasemlSeq:
     """Class representing parser for BASEML rst file.
 
     Methods for extracing tree, alignment, and residue probabilities for ancestral
@@ -254,11 +235,11 @@ class _Baseml_rstfile():
 
     ###############################################################################################
 
-    def __init__(self, filename=None):
+    def __init__(self, rstfile):
         """Check file contains needed info about tree and sequences.
         Construct dictionaries mapping between nodeID, seqname, and index"""
 
-        self.rstfile = open(filename, mode="rt", encoding="UTF-8")
+        self.rstfile = open(rstfile, mode="rt", encoding="UTF-8")
 
         self._read_until("^Ancestral reconstruction by BASEML")
         self.rstfile.readline()
@@ -283,6 +264,8 @@ class _Baseml_rstfile():
         # Note: nodenums start at 1, not 0 (this is BASEML node-numbering scheme)
         self.nodeid2nodenum, self.nodenum2nodeid = self._map_nodeid_nodenum()
         self.nodeids = set(self.nodeid2nodenum.keys())
+        self.tree = self.get_tree()
+        self.alignment, self.seqprob = self.get_alignment_info()
 
     ###########################################################################################
 
@@ -326,7 +309,7 @@ class _Baseml_rstfile():
 
     ###########################################################################################
 
-    def get_alignment(self):
+    def get_alignment_info(self):
         """Extracts sequence information corresponding to extant and ancestral sequences.
         Returns sequencelib.Seq_alignment object and 2D numpy array with residue probs"""
 
@@ -406,7 +389,7 @@ class _Baseml_rstfile():
 ###################################################################################################
 ###################################################################################################
 
-class _MBASR_file():
+class MBASRTrait:
     """Class representing parser for MBASR ancestral reconstruction file.
 
     Methods for extracing one tree and one alignment of ancestral (and contemporaneous) sequences.
@@ -430,7 +413,6 @@ class _MBASR_file():
 
         # Read treefile, store branch info as pandas dataframe
         # Note: I am cargo culting the rpy2 code here...
-        # Format:
         ape = importr("ape")
         tidytree = importr("tidytree")
         tree = ape.read_tree(treefile)
@@ -450,6 +432,8 @@ class _MBASR_file():
         self.nleaf = self.leafnodestate.shape[0]
         self.state0 = state0
         self.state1 = state1
+        self.tree = self.get_tree()
+        self.traitdict, self.traitprob = self.get_trait_dict()
 
     ###########################################################################################
 
@@ -474,12 +458,12 @@ class _MBASR_file():
     ###########################################################################################
 
     def get_trait_dict(self):
-        statenamedict = {0:self.state0, 1:self.state1}
+        traitnamedict = {0:self.state0, 1:self.state1}
         nodeidlist = list(self.leafnodestate["leafID"])
         intnodeidlist = [int(x.replace("node","")) for x in self.intnodestate["nodeID"]]
         nodeidlist.extend(intnodeidlist)
 
-        traitstatelist = [statenamedict[state] for state in self.leafnodestate["state"]]
+        traitstatelist = [traitnamedict[trait] for trait in self.leafnodestate["state"]]
         traitproblist = [1.0] * self.nleaf
         intnodestatelist = []
         intproblist = []
@@ -502,18 +486,7 @@ class _MBASR_file():
 ###################################################################################################
 ###################################################################################################
 
-class _TimeTreeResults:
-
-    def __init__(self, ancseqfile, seqtreefile, statetreefile, stateprobfile):
-        self.seqtree, self.seqname2id = self._parsetreefile(seqtreefile)
-        self.statetree, self.statename2id = self._parsetreefile(statetreefile)
-        self.alignment = self._parsealignfile(ancseqfile, self.seqtree, self.seqname2id)
-        self.check_input()
-        statename2seqid = self._match_state_seq(self.statetree, self.seqtree, self.statename2id)
-        self.trait, self.traitprob = self._parsetraits(statetreefile, stateprobfile,
-                                                        statename2seqid, self.seqtree)
-
-    ###########################################################################################
+class _TreeTime:
 
     def _parsetreefile(self, treefile):
         with pt.Nexustreefile(treefile) as tf:
@@ -538,6 +511,15 @@ class _TimeTreeResults:
         name2id[rootname] = tree.root
         return name2id
 
+###################################################################################################
+###################################################################################################
+
+class TreeTimeSeq(_TreeTime):
+
+    def __init__(self, ancseqfile, seq_treefile):
+        self.tree, self.seqname2id = self._parsetreefile(seq_treefile)
+        self.alignment = self._parsealignfile(ancseqfile, self.tree, self.seqname2id)
+
     ###########################################################################################
 
     def _parsealignfile(self, alignfile, tree, name2id):
@@ -549,28 +531,28 @@ class _TimeTreeResults:
             alignment.changeseqname(origname, newname)
         return alignment
 
-    ###########################################################################################
+###################################################################################################
+###################################################################################################
 
-    def _match_state_seq(self, statetree, seqtree, statename2id):
-        stateid2seqid,_,_ = statetree.match_nodes(seqtree)
-        statename2seqid = {}
-        for statename, stateid in statename2id.items():
-            seqid = stateid2seqid[stateid]
-            statename2seqid[statename] = seqid
-        return statename2seqid
+class TreeTimeTrait(_TreeTime):
+
+    def __init__(self, trait_treefile, trait_probfile):
+        self.tree, self.traitname2id = self._parsetreefile(trait_treefile)
+        self.traitdict, self.traitprob = self._parsetraits(trait_treefile, trait_probfile,
+                                                        self.tree, self.traitname2id)
 
     ###########################################################################################
 
-    def _parsetraits(self, statetreefile, stateprobfile, statename2seqid, seqtree):
-        trait = self._parse_annot_tree(statetreefile, statename2seqid) # This dict has no root entry
-        traitprob, seqid2AB = self._parse_CSV(stateprobfile, statename2seqid)
-        trait = self._match_AB_trait(seqid2AB, trait, seqtree)   # Add root trait. Check consistency
-        return trait, traitprob
+    def _parsetraits(self, trait_treefile, trait_probfile, trait_tree, traitname2id):
+        traitdict = self._parse_annot_tree(trait_treefile, traitname2id) # This dict has no root entry
+        traitprob, traitid2AB = self._parse_CSV(trait_probfile, traitname2id)
+        traitdict = self._match_AB_trait(traitid2AB, traitdict, trait_tree)   # Add root trait. Check consistency
+        return traitdict, traitprob
 
     ###########################################################################################
 
-    def _parse_annot_tree(self, statetreefile, statename2seqid):
-        with open(statetreefile, "r") as infile:
+    def _parse_annot_tree(self, trait_treefile, traitname2id):
+        with open(trait_treefile, "r") as infile:
             while True:
                 line = infile.readline()
                 if line.startswith("Begin Trees"):
@@ -584,41 +566,42 @@ class _TimeTreeResults:
                     treestring += line
         treestring = treestring.replace("\n", "")
         nodenames = re.findall(r"[\(\),]([^\(\),:]+):", treestring)
-        nodenames = nodenames[:-1]    # One too long: no state for root node for some reason
-        states = re.findall(r'\[&[\w]+=\"(\w+)\"\]', treestring)
+        nodenames = nodenames[:-1]    # One too long: no trait for root node for some reason
+        traits = re.findall(r'\[&[\w]+=\"(\w+)\"\]', treestring)
+
         traitdict = {}
-        for name,state in zip(nodenames, states):
-            seqid = statename2seqid[name]
-            traitdict[seqid] = state
+        for traitname, trait in zip(nodenames, traits):
+            traitid = traitname2id[traitname]
+            traitdict[traitid] = trait
         return traitdict
 
     ###########################################################################################
 
-    def _parse_CSV(self, stateprobfile, statename2seqid):
+    def _parse_CSV(self, trait_probfile, traitname2id):
         traitprob = {}
-        seqid2AB = {}
-        with open(stateprobfile) as csvfile:
+        traitid2AB = {}
+        with open(trait_probfile) as csvfile:
             reader = csv.reader(csvfile)
             next(reader)
             for row in reader:
-                seqid = statename2seqid[row[0]]
+                traitid = traitname2id[row[0]]
                 pA, pB = float(row[1]), float(row[2])
                 if pA > pB:
-                    traitprob[seqid] = pA
-                    seqid2AB[seqid] = "A"
+                    traitprob[traitid] = pA
+                    traitid2AB[traitid] = "A"
                 else:
-                    traitprob[seqid] = pB
-                    seqid2AB[seqid] = "B"
-        return traitprob, seqid2AB
+                    traitprob[traitid] = pB
+                    traitid2AB[traitid] = "B"
+        return traitprob, traitid2AB
 
     ###########################################################################################
 
-    def _match_AB_trait(self, seqid2AB, trait, seqtree):
+    def _match_AB_trait(self, traitid2AB, traitdict, trait_tree):
         # Python note: hardwired to two possible trait values. Think about generalizing
-        random_leaf = seqtree.leaflist()[0]
-        ABval = seqid2AB[random_leaf]
-        traitval = trait[random_leaf]
-        possible_traitvals = set(trait.values())
+        random_leaf = trait_tree.leaflist()[0]
+        ABval = traitid2AB[random_leaf]
+        traitval = traitdict[random_leaf]
+        possible_traitvals = set(traitdict.values())
         other_trait = (possible_traitvals - {traitval}).pop()
         AB2trait = {}
         if ABval == "A":
@@ -627,38 +610,22 @@ class _TimeTreeResults:
         else:
             AB2trait["A"] = other_trait
             AB2trait["B"] = traitval
-        root_ABtrait = seqid2AB[seqtree.root]
+        root_ABtrait = traitid2AB[trait_tree.root]
         root_trait = AB2trait[root_ABtrait]
-        trait[seqtree.root] = root_trait
-        self._check_trait_consistency(AB2trait, seqid2AB, trait)
-        return trait
+        traitdict[trait_tree.root] = root_trait
+        self._check_trait_consistency(AB2trait, traitid2AB, traitdict)
+        return traitdict
 
     ###########################################################################################
 
-    def _check_trait_consistency(self, AB2trait, seqid2AB, traitdict):
-        for seqid,trait in traitdict.items():
-            ABval = seqid2AB[seqid]
+    def _check_trait_consistency(self, AB2trait, traitid2AB, traitdict):
+        for traitid,traitval in traitdict.items():
+            ABval = traitid2AB[traitid]
             expected_trait = AB2trait[ABval]
-            if expected_trait != traitdict[seqid]:
-                raise AncError(f"Inconsistency between confidence.csv and annotated tree for node {seqid}:\n"
+            if expected_trait != traitdict[traitid]:
+                raise AncError(f"Inconsistency between confidence.csv and annotated tree for node {traitid}:\n"
                                f"Trait in CSV file:       {expected_trait}\n"
-                               f"Trait in annotated tree: {trait[seqid]}")
-
-    ###########################################################################################
-
-    def check_input(self):
-        if len(self.seqtree.nodes) != len(self.statetree.nodes):
-            raise AncError("State-tree and seq-tree do not have same number of nodes. Can't merge")
-        if self.seqtree.topology() != self.statetree.topology():
-            raise AncError("State-tree and seq-tree do not have same topology. Can't merge")
-        if len(self.alignment) != len(self.seqtree.nodes):
-            raise AncError(f"Wrong number of sequences:"
-                            " {len(self.alignment)} seqs in FASTA file, but"
-                            " {len(self.seqtree)} nodes in seqtree")
-
-
-
-
+                               f"Trait in annotated tree: {traitdict[traitid]}")
 
 
 
